﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WinGrid_Sample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            //Infragistics.Win.AppStyling.StyleManager.Load("../../IG.isl");
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // グリッドの初期設定
            ultraGrid1.DisplayLayout.Override.AllowRowFiltering = Infragistics.Win.DefaultableBoolean.True; //フィルタ設定
            ultraGrid1.DisplayLayout.Override.AllowGroupBy = Infragistics.Win.DefaultableBoolean.True; //グループ化設定
            ultraGrid1.DisplayLayout.Override.AllowColMoving = Infragistics.Win.UltraWinGrid.AllowColMoving.WithinBand; //列移動設定
            ultraGrid1.DisplayLayout.Override.AllowColSizing = Infragistics.Win.UltraWinGrid.AllowColSizing.Synchronized; //列サイズ設定
            
            // バインドするサンプルデータの設定
            DataTable dt = GetData();
            dt.AcceptChanges(); // バインド時にデータテーブルを全コミット
            this.ultraGrid1.DataSource = dt;

            // キー項目（Number列）を非表示化（もしくは編集不可へ）
            //ultraGrid1.DisplayLayout.Bands[0].Columns["Number"].Hidden = true;
            ultraGrid1.DisplayLayout.Bands[0].Columns["Number"].CellActivation = Infragistics.Win.UltraWinGrid.Activation.NoEdit; //もしくは編集不可        



            //最小値、最大値の設定（※数値項目、日付項目の場合のみ有効）
            ultraGrid1.DisplayLayout.Bands[0].Columns["Number"].MinValue = 0;
            ultraGrid1.DisplayLayout.Bands[0].Columns["Number"].MaxValue = 99999;

            //最大文字数（※文字列項目の場合のみ有効）
            ultraGrid1.DisplayLayout.Bands[0].Columns["String1"].MaxLength = 20;

            //正規表現を使った文字制限
            ultraGrid1.DisplayLayout.Bands[0].Columns["String1"].RegexPattern = "^[a-zA-Z0-9!-/:-@¥[-`{-~]+$";

            ultraGrid1.Validating += ultraGrid1_Validating;
            ultraGrid1.CellDataError += ultraGrid1_CellDataError;

            ultraGrid1.BeforeCellUpdate += ultraGrid1_BeforeCellUpdate;
            ultraGrid1.BeforeExitEditMode += ultraGrid1_BeforeExitEditMode;


            // DataErrorInfoを利用する。
            ultraGrid1.DisplayLayout.Override.SupportDataErrorInfo = Infragistics.Win.UltraWinGrid.SupportDataErrorInfo.RowsAndCells;

            //エラーセルの外観変更（）
            Infragistics.Win.Appearance ap = (Infragistics.Win.Appearance)ultraGrid1.DisplayLayout.Override.DataErrorCellAppearance;
            ap.BackGradientStyle = Infragistics.Win.GradientStyle.VerticalBump; //グラデーション設定
            ap.BackColor = Color.FromArgb(255, 181, 193); //ピンク
            ap.BackColor2 = Color.White; //白
        }

        //セルの編集から抜け出るときにチェック(この場合、エラーを改善しない限りフォーカスを抜けられない)
        void ultraGrid1_BeforeExitEditMode(object sender, Infragistics.Win.UltraWinGrid.BeforeExitEditModeEventArgs e)
        {
            //// String1列に"a"が含まれていないかどうかチェック
            //if (ultraGrid1.ActiveCell.Column.Key == "String1")
            //{
            //    string text = ultraGrid1.ActiveCell.EditorResolved.CurrentEditText;
            //    if (text.Contains("a"))
            //    {
            //        MessageBox.Show(text + " は \"a\" を含んでいるため入力できません。");
            //        e.Cancel = true;
            //    }
            //}
        }

        //セルの編集から抜け出た後にチェック(この場合、値が変更間のものに戻り、フォーカスは次へ遷移する)
        void ultraGrid1_BeforeCellUpdate(object sender, Infragistics.Win.UltraWinGrid.BeforeCellUpdateEventArgs e)
        {
            //// String1列に"a"が含まれていないかどうかチェック
            //if (e.Cell.Column.Key == "String1"){
            //    if(e.NewValue.ToString().Contains("a")){
            //        MessageBox.Show(e.NewValue.ToString() + " は \"a\" を含んでいるため入力できません。");
            //        e.Cancel = true;
            //    }
            //}

            // String1列に"a"が含まれていないかどうかチェック
            if (e.Cell.Column.Key == "String1")
            {
                if (e.NewValue.ToString().Contains("a"))
                {
                    //// セルにエラー貼り付け
                    //e.Cell.Row.DataErrorInfo.SetColumnError(
                    //    e.Cell.Column.Key,
                    //    e.Cell.Column.Key + " には \"a\" を入力できません。"
                    //    );
                    // セルにエラー貼り付け
                    DataRow row = ((DataRowView)e.Cell.Row.ListObject).Row;
                    row.SetColumnError(
                        e.Cell.Column.Key,
                        e.Cell.Column.Key + " には \"a\" を入力できません。"
                        );
                }
                else
                {
                    // セルにエラー解除
                    e.Cell.Row.DataErrorInfo.SetColumnError(e.Cell.Column.Key, null);
                }
            }
        }

        void ultraGrid1_CellDataError(object sender, Infragistics.Win.UltraWinGrid.CellDataErrorEventArgs e)
        {

        }

        void ultraGrid1_Validating(object sender, CancelEventArgs e)
        {
            
        }

        // *****************************
        // バインドするDatatableを生成
        // *****************************
        private DataTable GetData()
        {
            //Create DataTable
            DataTable dtSample1 = new DataTable("Sample");

            //Create Columns
            dtSample1.Columns.Add("Number", typeof(System.Int32));
            dtSample1.Columns.Add("Check", typeof(System.Boolean));
            dtSample1.Columns.Add("String1", typeof(System.String));
            dtSample1.Columns.Add("String2", typeof(System.String));
            dtSample1.Columns.Add("String3", typeof(System.String));
            dtSample1.Columns.Add("DateTime", typeof(System.DateTime));
            
            //Create Rows(20レコード生成する)
            for (int index = 0; index <= 19; index++)
            {
                dtSample1.Rows.Add(new object[] {
                                                    index,
                                                    false,
                                                    "String1_" + index.ToString(),
                                                    "String2_" + index.ToString(),
                                                    "String3_" + index.ToString(),
                                                    DateTime.Now.AddDays(1)
                                                });
            }
            //Number列をキーとする。
            dtSample1.PrimaryKey = new DataColumn[] { dtSample1.Columns["Number"] };

            return dtSample1;
        }

        // *****************************
        // 行追加ボタン押下
        // *****************************
        private void ultraButton1_Click(object sender, EventArgs e)
        {
            // バインドしている DataTable の取り出し
            DataTable dt = ultraGrid1.DataSource as DataTable;
            // 新規行追加 (キー項目だけは重複しないように注意)
            dt.Rows.Add(new object[] {dt.Rows.Count,false,"","",""});
            
        }

        // *****************************
        // 行削除ボタン押下
        // *****************************
        private void ultraButton2_Click(object sender, EventArgs e)
        {
            // 対象行チェック
            if (ultraGrid1.ActiveRow == null)
            {
                MessageBox.Show("削除対象行が選択されていません。");
                return;
            }

            int activeRowIndex = ultraGrid1.ActiveRow.Index;

            // 削除処理（自動的に確認ダイアログが出る。）
            ultraGrid1.ActiveRow.Delete();

            // アクティブ行を次の行へ移動
            if (activeRowIndex < ultraGrid1.Rows.Count)
                ultraGrid1.ActiveRow = ultraGrid1.Rows[activeRowIndex];
                        
        }

        // *****************************
        // 確定ボタン押下
        // *****************************
        private void ultraButton3_Click(object sender, EventArgs e)
        {

            // バインドしている DataTable の取り出し
            DataTable dt = ultraGrid1.DataSource as DataTable;

            // エラーが存在しないかどうか確認
            DataRow[] rows = dt.GetErrors();
            if (rows.Length != 0)
            {
                //存在する場合は先頭のエラー情報を取得
                String msg = rows[0].GetColumnError(rows[0].GetColumnsInError()[0]);
                MessageBox.Show(msg); //エラー表示
                return;
            }

            // DB更新確認
            if (MessageBox.Show(
                "編集内容を確定しますか？",
                "登録確認",
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Question)
                == DialogResult.No)
            {
                return;
            }
            
            // ・・・省略

            // 変更（更新、追加、削除）があった行のみの取り出し
            DataTable data = dt.GetChanges();

            if (data == null){
                MessageBox.Show("更新対象がありませんでした。");
                return;
            }
            
            foreach (DataRow row in data.Rows){
                switch (row.RowState) {
                    case DataRowState.Modified:
                        //更新処理へ
                        System.Diagnostics.Debug.WriteLine("RowModified Number:" + row["Number"].ToString());
                        break;
                    case DataRowState.Added:
                        //新規追加処理へ
                        System.Diagnostics.Debug.WriteLine("RowAdded Number:" + row["Number"].ToString());
                        break;
                    case DataRowState.Deleted:
                        //削除処理へ
                        System.Diagnostics.Debug.WriteLine("RowDeleted Number:" + row["Number",DataRowVersion.Original].ToString());
                        break;
                    default:
                        break;

                }
            }

            //更新処理完了
            dt.AcceptChanges();
            MessageBox.Show("更新しました。");
            
        }

        // *****************************
        // 元に戻すボタン押下
        // *****************************
        private void ultraButton4_Click(object sender, EventArgs e)
        {
            // バインドしている DataTable の取り出し
            DataTable dt = ultraGrid1.DataSource as DataTable;
            //元に戻す
            dt.RejectChanges();
        }
    }
}
